/**
 * ProjectDialog.java - Sets project properties within the Jipe Environment.
 *
 * Created by Steve Lawson. Copyright(c) 1996-2002.
 * Maintained by Jason Hardman. Copyright(c) 2002.
 * E-mail: elesario@users.sourceforge.net
 *
 * Homepage: http://jipe.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jipe;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

/** The Jipe project manager provides basic managment of a group of related files.
  */

/* CHANGED - 2002/05/03 : modified the way files are stored, within manager. Removed the
	vector and added the FileX tree nodes. Should make the manager more stable. Also made
	it impossible to save a project who's files haven't been saved also. -- Jason
*/
public class JipeProjectManager extends JDialog
{
    static String PROJECTNAME, PROJECTMAIN,SOURCEDIR,CLASSDIR,BUILDSCRIPT;
    boolean OPENPROJECT,SAVEPROJECT;
    static String name;
    static JTextField projectname;
    static JTextField projectmain;
    static JTextField classpath;
    static JTextField classdirectory;
    static JTextField buildscript;
    DefaultMutableTreeNode projecttreenode;
    TreeSelectionModel selectionModel;
    DefaultTreeModel model;
    static DefaultMutableTreeNode root;
    JTree projecttree;
    JFileChooser fileChooser;

    Container container;
    Jipe jipe;


    JipeProjectManager(Jipe child)
    {
		jipe = child;
		container = this.getContentPane();
		root = new DefaultMutableTreeNode("Root");
		projecttree = new JTree(root);
		model = (DefaultTreeModel)projecttree.getModel();
		selectionModel = projecttree.getSelectionModel();
		projecttree.setRootVisible(false);
		projecttree.setShowsRootHandles(false);
		JScrollPane projectScrollPane=new JScrollPane(projecttree);
		jipe.tabbedPane.addTab("Project", projectScrollPane);

		selectionModel.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		model.addTreeModelListener(new TreeModelListener(){

					  	public void treeNodesInserted(TreeModelEvent e)
					    {
						   	showInsertionOrRemoval(e, " added to ");
					    }
					    public void treeNodesRemoved(TreeModelEvent e)
					    {
						   	showInsertionOrRemoval(e, " removed from ");
					    }
					    private void showInsertionOrRemoval(TreeModelEvent e, String s)
					    {
						   	Object[] parentPath = e.getPath();
						   	int[] indexes = e.getChildIndices();
						   	Object[] children = e.getChildren();
						   	Object parent = parentPath[parentPath.length - 1];
						   	jipe.statusBar.msgline.setText("File: " + s);
					    }
					    public void treeNodesChanged(TreeModelEvent e)
					    {
					    }
					    public void treeStructureChanged(TreeModelEvent e)
					    {
					    }
					});

		projecttree.addMouseListener(new MouseAdapter(){
						public void mousePressed(MouseEvent e)
						{
						    JTree t = (JTree)e.getSource();
						    int row = t.getRowForLocation(e.getX(), e.getY());

						    TreePath path = t.getPathForLocation(e.getX(), e.getY());
						    DefaultMutableTreeNode treenode = null;
						    File file = null;
						    if (e.getClickCount() == 2)
						    {
								if (row != -1 && t == projecttree)
								{
								    treenode = (DefaultMutableTreeNode)path.getLastPathComponent();
								    if(treenode == null) return;
									if (treenode.isLeaf())
	                                {
										file = (File)treenode.getUserObject();
										if (jipe.GetWindow(file) == false)
											if(!file.exists()) return;
											else jipe.loadChild(file.getName(), file);
	                                }
								}
						    }
						}
					});
    }

    protected void addToProject()
    {
		if(projecttreenode != null)
		{
		    int index = projecttreenode.getChildCount();
		    File[] files = getAddChildFileName();
		    if(files!=null)
		    {
				for(int i=0; i < files.length; i++)
				{
				    MutableTreeNode newNode = new DefaultMutableTreeNode(new FileX(files[i]));
				    if(newNode!=null)
				    {
						model.insertNodeInto(newNode, projecttreenode, index+i);
						SAVEPROJECT=true;
				    }
				}
		    }
		}
    }

    protected void removeFromProject()
    {
		if (projecttreenode != null)
		{
		    TreePath path = selectionModel.getSelectionPath();
		    if (path.getPathCount() == 1)
		    {
				jipe.statusBar.msgline.setText("Can't remove root node!");
				return;
		    }
		    if (path.getPathCount() == 2)
		    {
				// removeProject();
				jipe.statusBar.msgline.setText("Select ' Close Project ' option from menubar");
				return;
		    }
		    if (path != null)
		    {
				MutableTreeNode node = (MutableTreeNode)path.getLastPathComponent();
				if (node.isLeaf())
				{
				    model.removeNodeFromParent(node);
				    SAVEPROJECT=true;
				}
		    }
		}
    }
    protected void ProjectProperties()
    {

		Font font = new Font("Dialog", Font.PLAIN, 11);
		setFont(font);
		setBackground(SystemColor.control);
		JPanel panel = new JPanel();
		GridLayout grid = new GridLayout(0, 2, 4, 0);
		panel.setBorder(
		    BorderFactory.createTitledBorder(""));
		panel.setLayout(grid);
		panel.add(new JLabel("Project Name : ", JLabel.CENTER));
		panel.add(projectname = new JTextField("", 15));
		panel.add(new JLabel("Main Class/Html File : ", JLabel.CENTER));
		panel.add(projectmain = new JTextField("", 15));
		panel.add(new JLabel("ClassPath : ", JLabel.CENTER));
		panel.add(classpath = new JTextField("", 15));
		panel.add(new JLabel("Class Directory : ", JLabel.CENTER));
		panel.add(classdirectory = new JTextField("", 15));
		panel.add(new JLabel("Build Script : ", JLabel.CENTER));
		panel.add(buildscript = new JTextField("", 15));
		getContentPane().add("North", panel);

		if(PROJECTNAME!=null){
		    projectname.setText(PROJECTNAME);
		    projectmain.setText(PROJECTMAIN);
		    classpath.setText(SOURCEDIR);
		    classdirectory.setText(CLASSDIR);
		    buildscript.setText(BUILDSCRIPT);
		}

		final JOptionPane pane= new JOptionPane(
					    panel, // message
					    JOptionPane.PLAIN_MESSAGE, // messageType
					    JOptionPane.OK_CANCEL_OPTION); // optionType


		JDialog dialog = pane.createDialog(
				     this, // parentComponent
				     "Project Properties"); // title

		dialog.pack();
		dialog.setResizable(false);
		dialog.show();

		Integer value=(Integer)pane.getValue();
		OPENPROJECT=true;
		if(value.intValue()==JOptionPane.OK_OPTION)UpdateReferences();
		if(value.intValue()==JOptionPane.CANCEL_OPTION)OPENPROJECT=false;


    }

    protected void OpenNewProject(File projectFile)
    {
		if (projecttreenode != null)CloseProject();
		if (projecttreenode == null)
		{
		    if(projectFile==null)return;
		    try
		    {
				jipe.statusBar.msgline.setText("Loading Project.");
				FileReader fis = new FileReader(projectFile.getPath());
				BufferedReader br = new BufferedReader(fis);

				PROJECTNAME= br.readLine();
				projecttreenode = new DefaultMutableTreeNode(PROJECTNAME);
				PROJECTMAIN = br.readLine();
				SOURCEDIR = br.readLine();
				CLASSDIR = br.readLine();
				BUILDSCRIPT = br.readLine();
				String str = br.readLine();
				while (str != null)
				{
					File file = new File(str);
//					if (str.endsWith("project"))
//						Jipe.projectFilePath = str;
				    projecttreenode.add(new DefaultMutableTreeNode(new FileX(file)));
				    str = br.readLine();
				}
				root.add(projecttreenode);
				model.reload(root);
				jipe.statusBar.msgline.setText("Project Loaded.");
				fis.close();

				// Expand tree upon creation
				projecttree.expandRow(0); // root node (find neater way, maybe TreePath?)

		    }
		    catch (Exception err)
		    {
				jipe.statusBar.msgline.setText("Error : " + err);
		    }
		}
    }

    protected void OpenNewProject()
    {
		if (projecttreenode != null)CloseProject();
		if (projecttreenode == null)
		{
		    File projectFile = getFileName("Open Project",3);
		    if(projectFile==null)return;
		    try
		    {
				jipe.statusBar.msgline.setText("Loading Project.");
				if(jipe.props.RPROJECTS.indexOf(projectFile.getAbsolutePath())<0)
				{
					jipe.props.RPROJECTS=projectFile.getAbsolutePath()+","+jipe.props.RPROJECTS;
					jipe.createRecentProjectsList();
				}
				FileReader fis = new FileReader(projectFile.getPath());
				BufferedReader br = new BufferedReader(fis);

				PROJECTNAME= br.readLine();
				projecttreenode = new DefaultMutableTreeNode(PROJECTNAME);
				PROJECTMAIN = br.readLine();
				SOURCEDIR = br.readLine();
				CLASSDIR = br.readLine();
				BUILDSCRIPT = br.readLine();
				String str = br.readLine();
				while (str != null)
				{
					File file=new File(str);
//					if (str.endsWith("project"))
//						Jipe.projectFilePath = str;
			    	projecttreenode.add(new DefaultMutableTreeNode(new FileX(file)));
			    	str = br.readLine();
				}
				root.add(projecttreenode);
				model.reload(root);
				jipe.statusBar.msgline.setText("Project Loaded.");
				fis.close();

				// Expand tree upon creation
				projecttree.expandRow(0); // root node (find neater way, maybe TreePath?)
		    }
		    catch (Exception err)
		    {
				jipe.statusBar.msgline.setText("Error : " + err);
		    }
		}
    }
    protected void CloseProject()
    {
		if(SAVEPROJECT)SaveConfirm("Save Project", "Project not Saved, Save Now?");
		else
		{
	    	this.dispose();
	    	jipe.tabbedPane.removeTabAt(0);
			jipe.CloseAll();
//			Jipe.projectFilePath = null;//added
	    	jipe.project=null;
		}
    }

    protected void NewProject()
    {
		if (projecttreenode != null)
			CloseProject();
		if (projecttreenode == null)
		{
		    ProjectProperties();
			if(OPENPROJECT)
			{
				projecttreenode = new DefaultMutableTreeNode(PROJECTNAME);
				root.add(projecttreenode);
				model.reload(root);

				// A file must be created here to so Jipe can be "aware" of it.
				// It hasn't been saved though and doesn't exist.
				File file = new FileX(PROJECTNAME+".project");
				MutableTreeNode newNode = new DefaultMutableTreeNode(file);
				model.insertNodeInto(newNode, projecttreenode, 0);
				jipe.addChild(jipe, file.getName(), file);

				// Expand tree upon creation
				projecttree.expandRow(0); // root node (find neater way, maybe TreePath?)
			}
		}
    }
    protected void SaveProject()
    {
		Vector x = new Vector();
		for(Enumeration children = projecttreenode.children(); children.hasMoreElements();)
        {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) children.nextElement();
			if(node.isLeaf())
			{
				File pf = (File) node.getUserObject();
            	if(!pf.exists())
            	{
	            	// ERROR: the user hasn't saved one of the files in the project
       				JOptionPane.showMessageDialog(getContentPane(),
       					pf.getName() + " is unsaved\nCannot save project file.",
       					"Error!",
       					JOptionPane.ERROR_MESSAGE);
	            	return;
            	}
				x.addElement(pf.getPath());
			}
        }

		File file = getFileName("Save Project",1);
		TreePath selPath;
		TreeNode node;

		if (file == null)return;
		else
		{
		    String filename=file.getPath();
		    if(!filename.endsWith(".prj"))filename=filename+".prj";
		    try
		    {
			FileWriter fs = new FileWriter(filename);
			fs.write(PROJECTNAME);
			fs.write(System.getProperty("line.separator"));
			fs.write(PROJECTMAIN);
			fs.write(System.getProperty("line.separator"));
			fs.write(SOURCEDIR);
			fs.write(System.getProperty("line.separator"));
			fs.write(CLASSDIR);
			fs.write(System.getProperty("line.separator"));
			fs.write(BUILDSCRIPT);
			fs.write(System.getProperty("line.separator"));
			Enumeration e=x.elements();
			while(e.hasMoreElements())
			    fs.write(e.nextElement() + System.getProperty("line.separator"));
			fs.close();
			SAVEPROJECT=false;
		    }
		    catch (Exception err)
		    {
		    }
		}
    }

    public void SaveConfirm(String confirmTitle, String confirmText)
    {
	int value = JOptionPane.showConfirmDialog(
			getContentPane(), confirmText, confirmTitle, JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
	switch (value)
	{
	case JOptionPane.YES_OPTION:
	    SaveProject();
	case JOptionPane.NO_OPTION:
	    this.dispose();
	    jipe.tabbedPane.removeTabAt(0);
			jipe.CloseAll();
	    jipe.project=null;
	case JOptionPane.CANCEL_OPTION:
	}
    }

    protected File getFileName(String title,int type){
	fileChooser = new JFileChooser(jipe.CURRENTDIR);
	fileChooser.setMultiSelectionEnabled(false);
	fileChooser.setDialogTitle(title);
	fileChooser.addChoosableFileFilter(new ProjectFilter());
	int selected=0;
	if(type==1)selected = fileChooser.showSaveDialog(container);
	else selected = fileChooser.showOpenDialog(container);
	if (selected == JFileChooser.APPROVE_OPTION){
	    jipe.CURRENTDIR=fileChooser.getCurrentDirectory();
	    return fileChooser.getSelectedFile();
	}
	else if (selected == JFileChooser. CANCEL_OPTION)
	    return null;
	return null;
    }

    protected File[] getAddChildFileName()
    {
		fileChooser = new JFileChooser(jipe.CURRENTDIR);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setDialogTitle("Add Files to Project");
		fileChooser.addChoosableFileFilter(new TextFilter());
		fileChooser.addChoosableFileFilter(new JspFilter());
		fileChooser.addChoosableFileFilter(new HtmlFilter());
		fileChooser.addChoosableFileFilter(new JavaFilter());
		fileChooser.addChoosableFileFilter(new ProjectFilter());
		fileChooser.setFileFilter(new JavaFilter());
		int selected = fileChooser.showOpenDialog(container);

		if(selected == JFileChooser.APPROVE_OPTION)
		{
		    jipe.CURRENTDIR=fileChooser.getCurrentDirectory();
		    return fileChooser.getSelectedFiles();
		}
		else if(selected == JFileChooser.CANCEL_OPTION)
		    return null;

		return null;
	}

	public void UpdateReferences()
	{
		PROJECTNAME=projectname.getText().trim();
		PROJECTMAIN=projectmain.getText().trim();
		SOURCEDIR=classpath.getText().trim();
		CLASSDIR=classdirectory.getText().trim();
		BUILDSCRIPT=buildscript.getText().trim();
		SAVEPROJECT=true;
    }

    void fileSaveUpdate(File oldFile, File newFile)
    {
	    for(Enumeration children = projecttreenode.children(); children.hasMoreElements();)
        {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) children.nextElement();
			if(node.isLeaf())
			{
				File f = (File) node.getUserObject();
				if(f.equals(oldFile))
					node.setUserObject(new FileX(newFile));
			}
        }
    }

    private class FileX extends java.io.File
    {
	    // Redeclare File's constructors and pass them up for default behaviour
	    public FileX(File parent, String child)
	    {
		    super(parent, child);
	    }

	    public FileX(String pathname)
	    {
		    super(pathname);
	    }

	    public FileX(String parent, String child)
	    {
		    super(parent, child);
	    }

	    public FileX(java.net.URI uri)
	    {
		    super(uri);
	    }

	    // A constructor that creates a FileX from a File
	    public FileX(File parent)
	    {
		    super(parent, "");
	    }

	    // The important change, toString now returns the filename, and not the path.
	    public String toString()
	    {
		    return getName();
	    }
    }
}


